;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xaef1287920bf0e0a81472de2bd8f12b9 . "./code.cg2js/cg-acache.fasl")
(#xa1e6366803b28db53c72609ecabee54a . "./code.cg2js/cg-base.fasl")
(#xfa8412a9af25e2bc45ec8796bc0eecc7 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xff0bad1b88e7c91537784d9d5adeeea2 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x95db3440431b7583287d86f3d71a11b0 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x474c652126c156fa8b6e5df7cd46f9ca . "./code.cg2js/cg-button.fasl")
(#xeb4009a49481c5cdf816a75651b55ace . "./code.cg2js/cg-calendar.fasl")
(#x672aaf49b21a8fb9d21c19cdb0918ccd . "./code.cg2js/cg-caret.fasl")
(#xaa05a4dfafbe09f4d1c1a1b0da80a79b . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa4decd185e21904a11d783c28fbbcafe . "./code.cg2js/cg-chart-widget.fasl")
(#x38bf9fb4124723b7a59af3e787c565bc . "./code.cg2js/cg-check-box.fasl")
(#xcf6dc8a6f373816be351a3e59d1b35f0 . "./code.cg2js/cg-choice-list.fasl")
(#xa2b37be1e8375f6622861aae9ca909b9 . "./code.cg2js/cg-choose-printer.fasl")
(#x1a1edd7075f7ffde5b4d5c74721f83c9 . "./code.cg2js/cg-class-grid.fasl")
(#x6a1844b0d1b8f08979b535ecbe46fd78 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x6c07fe792bd2c75ae60ea3bc58a3b5ac . "./code.cg2js/cg-class-support.fasl")
(#xaded539af4f6bbc41ce11442d47f4c28 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x018f5e763ca10e5e7fb7dc3f6601b30e . "./code.cg2js/cg-clipboard-stack.fasl")
(#xdf18b0f1c5c1133ea454319d72f37786 . "./code.cg2js/cg-clipboard.fasl")
(#x9256ca1c70f9947c387f8a0ea7027d93 . "./code.cg2js/cg-color-dialog.fasl")
(#x65af7b20399215e927e7e534d594b767 . "./code.cg2js/cg-combo-box.fasl")
(#x2f829dca9ebc762e95424a749bee7567 . "./code.cg2js/cg-common-control.fasl")
(#xbbe9f3e3a3705670191847d425783603 . "./code.cg2js/cg-comtab.fasl")
(#x7a8e0ce8cd1a2fc513103ac8ec6d69f7 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x9c7a481368f7ce4358585a1ddca40525 . "./code.cg2js/cg-curve.fasl")
(#x06f57497535e90f4de2922bd58d41568 . "./code.cg2js/cg-dde-utils.fasl")
(#xa9a53b016a52adadc772a57fdbea71cc . "./code.cg2js/cg-dev.fasl")
(#x5c6bd58e3230a6a37f932e244c994096 . "./code.cg2js/cg-dialog-item.fasl")
(#xb3861070a0d4fecc828810ea9f871546 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x03b3c310116d81208e5044c66ddbc237 . "./code.cg2js/cg-directory-dialog.fasl")
(#x56165c6302ed72ffb1c2c33f987e9b23 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x758920df08b2689ab06d9abbd79152c1 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x68a382a018572bfaad8f509868b9eea2 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x6deb4ce9be1e03e85f5595799dac1367 . "./code.cg2js/cg-drawable.fasl")
(#x3680fe79b963189328fb63d4b5ee582f . "./code.cg2js/cg-dropping-outline.fasl")
(#xc30a5e72e2d86cc6bca8ab1de821a633 . "./code.cg2js/cg-edit-in-place.fasl")
(#x76574dff503849e2f139df26abf04edf . "./code.cg2js/cg-editable-text.fasl")
(#xab34458716a0fb835f7e5c589a9816c5 . "./code.cg2js/cg-file-dialog.fasl")
(#x87f4179e81f9fb9eaf634678d6e647c2 . "./code.cg2js/cg-fill-texture.fasl")
(#x84d5ebe840c81082e10cd6344ad3378f . "./code.cg2js/cg-find-string-dialog.fasl")
(#xc0c36cb98402d37edff909d87f0d129d . "./code.cg2js/cg-font-dialog.fasl")
(#xf63c06a46cf4f0f65c8e29cca36c2c11 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x79242fb29ba8e46229f8d0a023b19453 . "./code.cg2js/cg-get-pixmap.fasl")
(#x237f32ace720f23bda6b765a111ae758 . "./code.cg2js/cg-get-position.fasl")
(#xbbfb2fd3bb7ccc0ec80569800f879f2f . "./code.cg2js/cg-graphics-context.fasl")
(#x3843d344591c3a27d6aba70283375673 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xbef94041cacd7e94142c8aae9ba051d7 . "./code.cg2js/cg-grid-widget.fasl")
(#xb7ca32b5e393dad58025773df919eb74 . "./code.cg2js/cg-group-box.fasl")
(#x56b7486cbc95e21a315fc5929bd24d47 . "./code.cg2js/cg-header-control.fasl")
(#x5fb9df51f72b5604367c16103c6f9282 . "./code.cg2js/cg-hotspot.fasl")
(#x11f54e7c22e4ed466ddc29fdd2d44c12 . "./code.cg2js/cg-html-dialog.fasl")
(#x15527fb7f77355c67f40518bafee4464 . "./code.cg2js/cg-html-widget.fasl")
(#xd65d4c514090f5bc141cb3cad83442bf . "./code.cg2js/cg-icon-pixmap.fasl")
(#x064d9cc5a4645ff1e1a58577dfe4eea2 . "./code.cg2js/cg-icon.fasl")
(#x16f1d789a32a284f2c81693c316b0627 . "./code.cg2js/cg-ie.fasl")
(#xb6e9ba6eae14d56967ef1b6edac63a4e . "./code.cg2js/cg-intersect-posbox.fasl")
(#xcc3f500248b25aa9bfb963333c7d1cbb . "./code.cg2js/cg-intersect.fasl")
(#x1bca947aac22431467e25d24bf4980cf . "./code.cg2js/cg-item-list.fasl")
(#x6be97244b8040ee51db9eb932a06a787 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x11536ef95d370c93c6cf7750f5be6ba1 . "./code.cg2js/cg-lamp.fasl")
(#xde559c3ef371d788678c63197084a259 . "./code.cg2js/cg-layout.fasl")
(#x487ac7dc689a25aecb1075ca2a73c84b . "./code.cg2js/cg-lettered-menu.fasl")
(#x29bd6299cbb4484a7c67689ae2364ed5 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x4237bc8054cb0adfe9dc908e80ea24e1 . "./code.cg2js/cg-lisp-text.fasl")
(#xa707829cf77d06eaa8ae7f64f2de6038 . "./code.cg2js/cg-lisp-widget.fasl")
(#x42c58ef68aca3e19e83c60a62ae63f41 . "./code.cg2js/cg-list-view.fasl")
(#x8d4d064e66662ba5a3d8f26c7d83e90e . "./code.cg2js/cg-mci.fasl")
(#x59e4527817bff53f8a878e9b5d640ed0 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x0d52fcb3bf9db71a630ba4613c45723d . "./code.cg2js/cg-menu.fasl")
(#x47583518591093427e23bd26d00b9416 . "./code.cg2js/cg-message-dialog.fasl")
(#x70789fa8c010f7e91f1da1fa4748306b . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xbee6e9a7a6033585acbad3bcc3dc634f . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x32ab70cadd3a1f05dccaddca1bdff7dd . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x3fbfa38c0eabc4edd883f22d007c5dd5 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x1f5f61ec7cec446c63e2a25869825b82 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x966d0b6764fc5d84a9365dee73658ddd . "./code.cg2js/cg-nodes.fasl")
(#xaa1277a4beb2308782002ee435a9f12c . "./code.cg2js/cg-object-editor-layout.fasl")
(#xd1acca682cd869b24c50c4c55f22c936 . "./code.cg2js/cg-object-editor.fasl")
(#xa83343256cfa9dfb3bad4141eff0dc65 . "./code.cg2js/cg-ocx.fasl")
(#x0c785f97e460109581e0cbf41680382c . "./code.cg2js/cg-os-widget.fasl")
(#x3229b24d7f45795a0977af4360f0e3a5 . "./code.cg2js/cg-os-window.fasl")
(#x7d34bf8286bfc0cc9e2e03ac34ca8f0c . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xc0222ceb3d113fb78c77d1fd929e4cde . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xeb13f1bd96b242eceb9593cc2f931704 . "./code.cg2js/cg-outline.fasl")
(#xa8636e577325f0921665efa7bb6b317f . "./code.cg2js/cg-palette.fasl")
(#x97a2ca85e4cd0d776255ebea8fbf62bc . "./code.cg2js/cg-paren-matching.fasl")
(#x222a3730d52280f07fa82fecbb3a4723 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x891bde49c31e50c4a7ee9758405ac73f . "./code.cg2js/cg-picture-widget.fasl")
(#x99bae3b24aff2ca59692913de6785e4f . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x7398ebf36867d07e4df9782c235868dc . "./code.cg2js/cg-pixmap-printing.fasl")
(#x0b64405366a21d439eb3d1c60a3ef363 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x78cfaed1ef91dd84c22e0d2a2dfafbd0 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x6592dcffc6870554233d92bb824beb70 . "./code.cg2js/cg-pixmap.fasl")
(#xcdaebab17c7bc2d8cb8c9a2c1a7a00bc . "./code.cg2js/cg-player.fasl")
(#x43f083b0ec02f5a92141b6360f430bb4 . "./code.cg2js/cg-plot-widget.fasl")
(#xd2ca04695471e90048b36f0185a4bd4a . "./code.cg2js/cg-printing.fasl")
(#x912fc01ea16995fc1360ef214c671a19 . "./code.cg2js/cg-progress-indicator.fasl")
(#xf10e221aa3bf9a9fc91ee4090c49e68d . "./code.cg2js/cg-project-window.fasl")
(#x2b0d584fdb1043bfaf09e40877e41d5e . "./code.cg2js/cg-property.fasl")
(#xbea996e053f8401231fc155f3e847664 . "./code.cg2js/cg-radio-button.fasl")
(#xe59fe40a158e3204f0e78cc352089ac2 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x2719385d5982f4ba038596e0635c96b6 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x64bbceb2b880f0d2f374e30e0455afbb . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x895e627a0a490149734d6277e6ada45d . "./code.cg2js/cg-rich-edit.fasl")
(#xd54005ba1ac356caea109510162be7ff . "./code.cg2js/cg-sample-file-menu.fasl")
(#xc25308a0b71b3194010b68a4afe8db14 . "./code.cg2js/cg-scaling-stream.fasl")
(#x051c93f7d8669a2abbcef469297f9057 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xebfec5b29021b1ff0f7e52ad7d10be23 . "./code.cg2js/cg-scroll-bar.fasl")
(#x98d67d90e7523e326636636abc1767e0 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x71e983f9ef3c9d880b1e32b25c377780 . "./code.cg2js/cg-selected-object.fasl")
(#x9c26b3e8c2472e9688fea7fd6fe8cf7a . "./code.cg2js/cg-shortcut-menu.fasl")
(#x7beced057ded5f79a7107c62c77542b3 . "./code.cg2js/cg-split-bar.fasl")
(#x2089c3b2378d010d4a645abb54397bbb . "./code.cg2js/cg-static-text.fasl")
(#xf9bfb6075f3fa30bdd7ff70b305f7dfc . "./code.cg2js/cg-status-bar.fasl")
(#x1dc2371f7bd22606637a4fc1f8741a0e . "./code.cg2js/cg-string-dialog.fasl")
(#xaec744247fb9cdce41c978af77c54643 . "./code.cg2js/cg-tab-control.fasl")
(#x9e7a24752c8508502d55d1cf52f10cf0 . "./code.cg2js/cg-template-string.fasl")
(#x001e38d6827599d0eab9ed3ee9d6b90f . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x5daf13efb187eeae30ea20741013cbbd . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x0e311d1a8801c846f49ef645495c7cb2 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xd32511d6cd7d5ff8e5bc118dc4c83a38 . "./code.cg2js/cg-text-or-combo.fasl")
(#x2752efa864850be916155cb96d4ed861 . "./code.cg2js/cg-text-widget.fasl")
(#x4a7ec29cccd6e847b63b47361f3f4d31 . "./code.cg2js/cg-timer.fasl")
(#xb480b2ef0cd20b69392d079dc1bc75a8 . "./code.cg2js/cg-toggling-widget.fasl")
(#xb3515bf2241b540d7274461b1f295790 . "./code.cg2js/cg-toolbar.fasl")
(#xad1de1d287225a17c43df558f28b1dad . "./code.cg2js/cg-tooltip.fasl")
(#x037aed5842fc11f5b8cce6b99364042f . "./code.cg2js/cg-touch.fasl")
(#x1e95398bbdb2656352d194dc9f5e5fe2 . "./code.cg2js/cg-trackbar.fasl")
(#x179c642913adb2c9b53467a0fe5494fa . "./code.cg2js/cg-tray.fasl")
(#x9bb72817f5f45ccd3eaa820fc4d1bfe2 . "./code.cg2js/cg-up-down-control.fasl")
(#x8ccf37948b8c1b3948bfc037e44a2c06 . "./code.cg2js/cg-utility-dialog.fasl")
(#x709b6582f6eabc1539cb209019d84dc3 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x6f2a786368b1651e847f481804983c72 . "./code.cg2js/cg-web-browser.fasl")
(#xf3d1cf54bd5357ef7763b04e37f22356 . "./code.cg2js/cg-wrap-string.fasl")
(#x99ccf18e388145be1afd333d13e0a9ec . "./code.cg2js/cg-yes-no-list.fasl")
(#x5501758e92dad07cf7964f0f9ba053d5 . "./code.cg2js/cg.fasl")
(#xe33d6cc883255b5580062d232f4f3e98 . "./code.cg2js/cg-yes-no-string.fasl")
(#xd4f981947896adb4eda13d0e3ca0aef0 . "./code.idejs/ide-acache.fasl")
(#x0910a7f5b1ec7925d504a8b128efbf37 . "./code.idejs/ide-apropos.fasl")
(#x1340ec8fad5489b9af7cfb6d86980891 . "./code.idejs/ide-base.fasl")
(#x0b1a7182baa87cab131c8aa8f221ee16 . "./code.idejs/ide-class-browser.fasl")
(#xa7ab7f865d4bb251109fcbe698fdb068 . "./code.idejs/ide-class-grapher.fasl")
(#x7bf00c85bd9fa0e64c1be5c5676ed34a . "./code.idejs/ide-clipboard.fasl")
(#x0ba0c4b632a6a5b1cf5598a585460ddb . "./code.idejs/ide-dev.fasl")
(#x150e2ac68a918ebcd9883441b924072f . "./code.idejs/ide-editor.fasl")
(#x4d5519e93199ba19239d969b13be9717 . "./code.idejs/ide-find-definitions.fasl")
(#x9021d1451ed60904350bd87d686ca170 . "./code.idejs/ide-find-in-files.fasl")
(#xa5e6545ecc33bb59540a58e5705b8af1 . "./code.idejs/ide-form.fasl")
(#x3f19a4bf83219cfab2d0833a402a6af2 . "./code.idejs/ide-grapher.fasl")
(#x9524fbf9af136851ff05c7822d969be3 . "./code.idejs/ide-inspector.fasl")
(#x31fff50d7c327a3eaea593654adf4231 . "./code.idejs/ide-intro.fasl")
(#x394bbd3d4644bbcd1b9664ec4c5cd269 . "./code.idejs/ide-keyshort.fasl")
(#xda25399411795b090cd187e9526a762f . "./code.idejs/ide-navigator.fasl")
(#x1d20d40696c3497a2c78b057cb7b3b1d . "./code.idejs/ide-package-list.fasl")
(#x311d1233ec60737edb4a702690c05afa . "./code.idejs/ide-patch-dialog.fasl")
(#xcca9b01018fe11310fbed9f3074a2794 . "./code.idejs/ide-process-browser.fasl")
(#x2465f8ec68581169ede2e282442050d7 . "./code.idejs/ide-project.fasl")
(#xe32add8324495a407213b13ed2722f8f . "./code.idejs/ide-trace.fasl")
(#xb495e0b3dab491dc5cfd039ec1b9c8b3 . "./code.idejs/ide-window-list.fasl")
(#x5ff4b6a04182767fb952c4f8cc0366ab . "./code.idejs/ide.fasl")
(#xf2773f648685397c1e60fb942db24e35 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
